//Include the NeoKey library
#include "Adafruit_NeoKey_1x4.h"
#include "seesaw_neopixel.h"

// Include the AccelStepper library:
#include <AccelStepper.h>

#define Y_DIM 2  //number of rows of keys
#define X_DIM 4  //number of columns of keys

#define dirPin 8
#define stepPin 9
#define enablePin 7
#define motorInterfaceType 1

#define myfullrevolution 6400  // how many stepes for the motor to make a full revolution - change to match the setting on your stepper driver
#define mymovetendegrees 18

int haveButtons = false;

// Create a new instance of class stepper motor and the seesaw (button controller):

AccelStepper stepper = AccelStepper(motorInterfaceType, stepPin, dirPin);


// Create a matrix of NeoKey 1x4's
Adafruit_NeoKey_1x4 nk_array[Y_DIM][X_DIM / 4] = {
  { Adafruit_NeoKey_1x4(0x30) },
  { Adafruit_NeoKey_1x4(0x31) },
};

// pass the matrix to the multi-neokey object
Adafruit_MultiNeoKey1x4 neokey((Adafruit_NeoKey_1x4 *)nk_array, Y_DIM, X_DIM / 4);

#define Black  0x000000
#define White 0xFFFFFF
#define Red 0xFF0000
#define Lime  0x00FF00
#define Blue  0x0000FF
#define Yellow  0xFFFF00
#define Cyan  0x00FFFF
#define Magenta  0xFF00FF
#define Silver  0xC0C0C0
#define Gray  0x808080
#define Maroon  0x800000
#define Olive 0x808000
#define Green 0x008000
#define Purple  0x800080
#define Teal  0x008080
#define Navy  0x000080



void setup() {
  Serial.begin(115200);
  while (!Serial) delay(10);

  Serial.println(F("All the world is a stage, and we are its actors."));  // quick message to let us know that the serial driver is up

  if (!neokey.begin()) {  // start matrix
    Serial.println("Could not start NeoKeys, check wiring?");
    haveButtons = false;
    //while (1) delay(10);

  } else {

    haveButtons = true;


    if (haveButtons == true) {
      Serial.println("NeoKey started!");
      Serial.println("My Life is compleat, I recognize the buttons are there.");


      // Pulse all the LEDs on to show we're working

      for (uint16_t i = 0; i < X_DIM * Y_DIM; i++) {
        neokey.setPixelColor(i, 0x808080);  // make each LED white
        neokey.show();
        delay(50);
      }

      for (uint16_t i = 0; i < X_DIM * Y_DIM; i++) {
        neokey.setPixelColor(i, 0x000000);
        neokey.show();
        delay(50);
      }

      // activate all keys and set callbacks
      for (int y = 0; y < Y_DIM; y++) {
        for (int x = 0; x < X_DIM; x++) {
          neokey.registerCallback(x, y, blink);
        }
      }
      // add some color to the party
      neokey.setPixelColor(0, Purple);
      neokey.setPixelColor(1, Red);
      neokey.setPixelColor(2, Green);
      neokey.setPixelColor(3, Purple);
      neokey.setPixelColor(4, Olive);
      neokey.setPixelColor(5, Blue);
      neokey.setPixelColor(6, Blue);
      neokey.setPixelColor(7, Olive);


    }

    // Set the maximum speed and acceleration:
    stepper.setEnablePin(enablePin);
    stepper.setPinsInverted(false, false, true);  // this sets the enable pin if you are using it.
    stepper.enableOutputs();
    stepper.setMaxSpeed(1000);
    stepper.setAcceleration(250);
    stepper.disableOutputs();
  }


  Serial.println(F("End of startup"));  // quick message to let us know that the serial driver is up
}

void loop() {
  if (haveButtons == false) {

    Serial.println("I do not have buttons and i am sad");

    stepper.setMaxSpeed(1000);
    stepper.setAcceleration(100);
    stepper.enableOutputs();
    stepper.move(myfullrevolution * 4);
    stepper.runToPosition();
    stepper.move(-myfullrevolution * 4);
    stepper.runToPosition();
    stepper.setMaxSpeed(500);
    stepper.setAcceleration(50);
    stepper.disableOutputs();
    delay(1000);

  } else {

    neokey.read();

    delay(10);  // don't print too fast
  }
}


//define a callback for key presses
NeoKey1x4Callback blink(keyEvent evt) {
  uint8_t key = evt.bit.NUM;

  if (evt.bit.EDGE == SEESAW_KEYPAD_EDGE_RISING) {
    Serial.print("Key press ");
    Serial.println(key);
    neokey.setPixelColor(key, Black);
    //Serial.println("My Life is compleat, I recognize the buttons are there.");

    if (key == 0) {
      Serial.println("Switch 0 pressed, half revolution CW");
      
      stepper.enableOutputs();
      stepper.move(myfullrevolution / 2);
      stepper.runToPosition();
      stepper.disableOutputs();
    }

    if (key == 1) {
      Serial.println("Switch 1 pressed, move to zero");
     
      stepper.enableOutputs();
      stepper.moveTo(0);
      stepper.runToPosition();
      stepper.disableOutputs();
    }

    if (key == 2) {
      Serial.println("Switch 2 pressed, move a full revolution");

      stepper.enableOutputs();
      stepper.move(myfullrevolution);
      stepper.runToPosition();
      stepper.disableOutputs();
    }

    if (key == 3) {
      Serial.println("Switch 3 pressed,half revolution CCW");

      stepper.enableOutputs();
      stepper.move(-myfullrevolution / 2);
      stepper.runToPosition();
      stepper.disableOutputs();
    }

    if (key == 4) {
      Serial.println("Switch 4 pressed, move 5 CW");

      stepper.enableOutputs();
      stepper.move(mymovetendegrees * 5);
      stepper.runToPosition();
      stepper.disableOutputs();
    }

    if (key == 5) {
      Serial.println("Switch 5 pressed, quater revolution CW");   

      stepper.enableOutputs();
      stepper.move(myfullrevolution / 4);
      stepper.runToPosition();
      stepper.disableOutputs();
    }

    if (key == 6) {
      Serial.println("Switch 6 pressed, quarter revolution CCW");
      
      stepper.enableOutputs();
      stepper.move(-myfullrevolution / 4);
      stepper.runToPosition();
      stepper.disableOutputs();
    }

    if (key == 7) {
      Serial.println("Switch 7 pressed, move 5 degrees CCW");

      stepper.enableOutputs();
      stepper.move(-mymovetendegrees * 5);
      stepper.runToPosition();
      stepper.disableOutputs();
    }

  } else if (evt.bit.EDGE == SEESAW_KEYPAD_EDGE_FALLING) {
    Serial.print("Key release ");
    Serial.println(key);

    //neokey.setPixelColor(key, 0);
    neokey.setPixelColor(0, Purple);
    neokey.setPixelColor(1, Red);
    neokey.setPixelColor(2, Green);
    neokey.setPixelColor(3, Purple);
    neokey.setPixelColor(4, Olive);
    neokey.setPixelColor(5, Blue);
    neokey.setPixelColor(6, Blue);
    neokey.setPixelColor(7, Olive);



  }

  // Turn on/off the neopixels!
  neokey.show();
  return 0;
}

/******************************************/
